@@ -1,15 +1,14 @@
-#' LASSO
+#' Least Absolute Shrinkage and Selection Operator
 #'
 #' LASSO, or L1-regularized regression, is an optimization problem to solve
-#' \deqn{min_x ~ \frac{1}{2}\|Ax-b\|_2^2 + \lambda \|x\|_1}
+#' \deqn{\textrm{min}_x ~ \frac{1}{2}\|Ax-b\|_2^2 + \lambda \|x\|_1}
 #' for sparsifying the coefficient vector \eqn{x}.
 #' The implementation is borrowed from Stephen Boyd's
 #' \href{http://stanford.edu/~boyd/papers/admm/lasso/lasso.html}{MATLAB code}.
 #'
 #' @param A an \eqn{(m\times n)} regressor matrix
 #' @param b a length-\eqn{m} response vector
-#' @param lambda a regularization parameber
-#' @param xinit a length-\eqn{n} vector for initial value
+#' @param lambda a regularization parameter
 #' @param rho an augmented Lagrangian parameter
 #' @param alpha an overrelaxation parameter in [1,2]
 #' @param abstol absolute tolerance stopping criterion
@@ -36,8 +35,8 @@
 #'
 #' @examples
 #' ## generate sample data
-#' m = 500
-#' n = 1000
+#' m = 50
+#' n = 100
 #' p = 0.1   # percentange of non-zero elements
 #'
 #' x0 = matrix(Matrix::rsparsematrix(n,1,p))
@@ -65,8 +64,7 @@
 #'
 #' @rdname LASSO
 #' @export
-admm.lasso <- function(A, b, lambda=1.0, xinit=NA,
-                    rho=1.0, alpha=1.0,
+admm.lasso <- function(A, b, lambda=1.0, rho=1.0, alpha=1.0,
                     abstol=1e-4, reltol=1e-2, maxiter=1000){
   ## PREPROCESSING
   # data validity
@@ -79,18 +77,33 @@ admm.lasso <- function(A, b, lambda=1.0, xinit=NA,
   if (nrow(A)!=length(b)){
     stop("* ADMM.LASSO : two inputs 'A' and 'b' have non-matching dimension.")}
   # initial value
-  if (!is.na(xinit)){
-    if ((!check_data_vector(xinit))||(length(xinit)!=ncol(A))){
-      stop("* ADMM.LASSO : input 'xinit' is invalid.")
-    }
-    xinit = as.vector(xinit)
-  } else {
-    xinit = as.vector(rep(0,ncol(A)))
-  }
+  xinit = as.vector(rnorm(ncol(A))/10)
+  # if (!is.na(xinit)){
+  #   if ((!check_data_vector(xinit))||(length(xinit)!=ncol(A))){
+  #     stop("* ADMM.LASSO : input 'xinit' is invalid.")
+  #   }
+  #   xinit = as.vector(xinit)
+  # } else {
+  #   xinit = as.vector(rep(0,ncol(A)))
+  # }
+
+
   # other parameters
-  if (!check_param_constant(lambda)){
+  meps = (.Machine$double.eps)
+  negsmall = -meps
+  if (!check_param_constant(lambda,negsmall)){
     stop("* ADMM.LASSO : reg. parameter 'lambda' is invalid.")
   }
+  if (lambda < meps){
+    message("* ADMM.LASSO : since 'lambda' is effectively zero, a least-squares solution is returned.")
+    xsol   = as.vector(aux_pinv(A)%*%matrix(b))
+    output = list()
+    output$x = xsol
+    return(output)
+  }
+
+
+
   if (!check_param_constant_multiple(c(abstol, reltol))){
     stop("* ADMM.LASSO : tolerance level is invalid.")
   }
@@ -108,6 +121,10 @@ admm.lasso <- function(A, b, lambda=1.0, xinit=NA,
     warning("* ADMM.LASSO : 'alpha' value is suggested to be in [1,2].")
   }
 
+
+
+
+
   ## MAIN COMPUTATION & RESULT RETURN
   result = admm_lasso(A,b,lambda,xinit,reltol,abstol,maxiter,rho,alpha)
 
